#
# Copyright (C) 2025 Masaryk University
#
# MU-INVENIO-CLI is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.
"""CLI context storage for the MU Invenio CLI application."""

from mu_invenio_cli.states.state import State


class CLIContext:
    def __init__(self):
        self.config = {}
        self.selected_id = None
        self.json_body = None
        self.selected_data = None
        self.state = State.CONFIGURATION
        self.last_state = State.CONFIGURATION



#
# Copyright (C) 2025 Masaryk University
#
# MU-INVENIO-CLI is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.
"""Help messages for the MU Invenio CLI application."""

def help_config_state():
    print("\nConfiguration Help:")
    print(f"BASE_API_URL - The base URL for the API. Example: https://dar.elter-ri.eu/api")
    print(f"API_TOKEN - The API token for the API. This token can be generated in the Profile Application sections in the DAR")
    print(f"MODEL - The model to use for the API. Example: datasets/external-datasets")


#
# Copyright (C) 2025 Masaryk University
#
# MU-INVENIO-CLI is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.
"""State management enum for the MU Invenio CLI application."""

from enum import Enum


class State(Enum):
    CONFIGURATION = 1
    NOT_SELECTED = 2
    SELECTED = 3
    HELP = 4



#
# Copyright (C) 2025 Masaryk University
#
# MU-INVENIO-CLI is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.
#
"""Base abstract state class for the MU Invenio CLI application."""


from abc import ABC, abstractmethod


class BaseState(ABC):
    def __init__(self, context):
        self.context = context

    @abstractmethod
    def handle(self):
        pass



#
# Copyright (C) 2025 Masaryk University
#
# MU-INVENIO-CLI is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.
#
"""Test if the app can run without error."""

import unittest
from unittest.mock import patch
from mu_invenio_cli import main as cli_main
from mu_invenio_cli.cli_context import CLIContext

class TestMainEntryPoint(unittest.TestCase):
    @patch("builtins.input", side_effect=["0"])
    @patch("sys.exit", side_effect=SystemExit)
    def test_main_exits_from_configuration(self, mock_exit, mock_input):
        with patch("builtins.print"):
            with self.assertRaises(SystemExit) as cm:
                cli_main.main()
        self.assertEqual(cm.exception.code, 0)
        ctx = CLIContext()
        self.assertEqual(ctx.state.name, "CONFIGURATION")


#
# Copyright (C) 2025 Masaryk University
#
# MU-INVENIO-CLI is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.
#
"""Selected state for the MU Invenio CLI application.
This state handles the actions when the draft is selected.
"""

from .base_state import BaseState


class SelectedState(BaseState):
    def handle(self):
        print("Selected State")



#
# Copyright (C) 2025 Masaryk University
#
# MU-INVENIO-CLI is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.
#

"""Helper state for the MU Invenio CLI application.
This state provides help options for the user.
"""

from mu_invenio_cli.help_messages import help_config_state
from mu_invenio_cli.states.base_state import BaseState


class HelperState(BaseState):

    def handle(self):
        print("\nHelp:")
        while True:
            print("\nOptions:")
            print("  1. Configuration help")
            print("  9. Back")
            print("  0. Exit")

            choice = input("Select option: ").strip()
            if choice == '1':
                help_config_state()
            elif choice == '9':
                self.context.state = self.context.last_state
                break
            elif choice == '0':
                exit(0)
            else:
                print("Invalid option. Please try again.")
